@@ -1,5 +1,6 @@
 import os
 import json
+import logging
 
 import singer
 import singer.bookmarks as bookmarks
@@ -15,6 +16,7 @@ from .tableau.tasks import get_all_task_details
 from .tableau.workbooks import get_all_workbook_details
 
 
+logging.disable(30) # Turn off logging from client library
 logger = singer.get_logger()
 
 REQUIRED_CONFIG_KEYS = ['host']
@@ -39,7 +41,7 @@ def get_bookmark(state, stream_name, bookmark_key, start_date):
     return None
 
 
-def get_all_datasources(schema, server, authentication, state, mdata, start_date):
+def get_all_datasources(schema, server_client, state, mdata, start_date):
     bookmark_value = get_bookmark(state, 'datasources', 'updated_at', start_date)
     if bookmark_value:
         max_record_value = singer.utils.strptime_to_utc(bookmark_value)
@@ -47,27 +49,27 @@ def get_all_datasources(schema, server, authentication, state, mdata, start_date
         max_record_value = singer.utils.strptime_to_utc("1970-01-01")
     with metrics.record_counter('datasources') as counter:
         extraction_time = singer.utils.now()
-        datasource_details = get_all_datasource_details(server=server, authentication=authentication, start_date=max_record_value)
+        datasource_details = get_all_datasource_details(server_client=server_client, start_date=max_record_value)
         for datasource in datasource_details['datasources']:
             with singer.Transformer() as transformer:
                 rec = transformer.transform(datasource, schema, metadata=metadata.to_map(mdata))
             singer.write_record('datasources', rec, time_extracted=extraction_time)
+            if singer.utils.strptime_to_utc(rec['updated_at']) > max_record_value:
+                max_record_value = singer.utils.strptime_to_utc(rec['updated_at'])
             counter.increment()
             if schema.get('connections'):
                 for connection in datasource_details['connections']:
                     with singer.Transformer() as transformer:
                         rec = transformer.transform(connection, schema, metadata=metadata.to_map(mdata))
                     singer.write_record('connections', rec, time_extracted=extraction_time)
-            if singer.utils.strptime_to_utc(rec['updated_at']) > max_record_value:
-                max_record_value = singer.utils.strptime_to_utc(rec['updated_at'])
     state = singer.write_bookmark(state, 'datasources', 'updated_at', singer.utils.strftime(max_record_value))
     return state
 
 
-def get_all_groups(schema, server, authentication, state, mdata, _start_date):
+def get_all_groups(schema, server_client, state, mdata, _start_date):
     with metrics.record_counter('groups') as counter:
         extraction_time = singer.utils.now()
-        group_details = get_all_group_details(server=server, authentication=authentication)
+        group_details = get_all_group_details(server_client=server_client)
         for group in group_details['groups']:
             with singer.Transformer() as transformer:
                 rec = transformer.transform(group, schema, metadata=metadata.to_map(mdata))
@@ -81,10 +83,10 @@ def get_all_groups(schema, server, authentication, state, mdata, _start_date):
     return state
 
 
-def get_all_projects(schema, server, authentication, state, mdata, _start_date):
+def get_all_projects(schema, server_client, state, mdata, _start_date):
     with metrics.record_counter('projects') as counter:
         extraction_time = singer.utils.now()
-        project_details = get_all_project_details(server=server, authentication=authentication)
+        project_details = get_all_project_details(server_client=server_client)
         for project in project_details:
             with singer.Transformer() as transformer:
                 rec = transformer.transform(project, schema, metadata=metadata.to_map(mdata))
@@ -93,10 +95,10 @@ def get_all_projects(schema, server, authentication, state, mdata, _start_date):
     return state
 
 
-def get_all_schedules(schema, server, authentication, state, mdata, _start_date):
+def get_all_schedules(schema, server_client, state, mdata, _start_date):
     with metrics.record_counter('schedules') as counter:
         extraction_time = singer.utils.now()
-        schedule_details = get_all_schedule_details(server=server, authentication=authentication)
+        schedule_details = get_all_schedule_details(server_client=server_client)
         for schedule in schedule_details:
             with singer.Transformer() as transformer:
                 rec = transformer.transform(schedule, schema, metadata=metadata.to_map(mdata))
@@ -105,10 +107,10 @@ def get_all_schedules(schema, server, authentication, state, mdata, _start_date)
     return state
 
 
-def get_all_tasks(schema, server, authentication, state, mdata, _start_date):
+def get_all_tasks(schema, server_client, state, mdata, _start_date):
     with metrics.record_counter('tasks') as counter:
         extraction_time = singer.utils.now()
-        task_details = get_all_task_details(server=server, authentication=authentication)
+        task_details = get_all_task_details(server_client=server_client)
         for task in task_details:
             with singer.Transformer() as transformer:
                 rec = transformer.transform(task, schema, metadata=metadata.to_map(mdata))
@@ -117,7 +119,7 @@ def get_all_tasks(schema, server, authentication, state, mdata, _start_date):
     return state
 
 
-def get_all_workbooks(schema, server, authentication, state, mdata, start_date):
+def get_all_workbooks(schema, server_client, state, mdata, start_date):
     bookmark_value = get_bookmark(state, 'workbooks', 'updated_at', start_date)
     if bookmark_value:
         max_record_value = singer.utils.strptime_to_utc(bookmark_value)
@@ -125,7 +127,7 @@ def get_all_workbooks(schema, server, authentication, state, mdata, start_date):
         max_record_value = singer.utils.strptime_to_utc("1970-01-01")
     with metrics.record_counter('workbooks') as counter:
         extraction_time = singer.utils.now()
-        workbook_details = get_all_workbook_details(server=server, authentication=authentication, start_date=max_record_value)
+        workbook_details = get_all_workbook_details(server_client=server_client, start_date=max_record_value)
         for workbook in workbook_details['workbooks']:
             with singer.Transformer() as transformer:
                 rec = transformer.transform(workbook, schema, metadata=metadata.to_map(mdata))
@@ -166,6 +168,25 @@ def load_schemas():
     return schemas
 
 
+class DependencyException(Exception):
+    pass
+
+
+def validate_dependencies(selected_stream_ids):
+    errs = []
+    msg_tmpl = ("Unable to extract '{0}' data, "
+                "to receive '{0}' data, you also need to select '{1}'.")
+
+    for main_stream, sub_streams in SUB_STREAMS.items():
+        if main_stream not in selected_stream_ids:
+            for sub_stream in sub_streams:
+                if sub_stream in selected_stream_ids:
+                    errs.append(msg_tmpl.format(sub_stream, main_stream))
+
+    if errs:
+        raise DependencyException(" ".join(errs))
+
+
 def populate_metadata(schema_name, schema):
     mdata = metadata.new()
     mdata = metadata.write(mdata, (), 'table-key-properties', KEY_PROPERTIES[schema_name])
@@ -219,45 +240,46 @@ def get_stream_from_catalog(stream_id, catalog):
     return None
 
 
-def discover(config):
+def discover():
     catalog = get_catalog()
     print(json.dumps(catalog, indent=2))
 
 
 def do_sync(config, state, catalog):
     if config.get('username') and config.get('password'):
-        print("Using username/ password based authentication")
         authentication = TSC.TableauAuth(config['username'], config['password'], site_id=config.get('site_id'))
     elif config.get('token_name') and config.get('token'):
-        print("Using token based authentication")
         authentication = TSC.PersonalAccessTokenAuth(config['token_name'], config['token'], site_id=config.get('site_id'))
     else:
         raise ValueError("Must specify username/ password or token_name/ token for authentication")
-    server = TSC.Server(config['host'], use_server_version=True)
+
+    server_client = TSC.Server(config['host'], config['server_version']) if config.get('server_version') else TSC.Server(config['host'], use_server_version=True)
+
+    if not server_client.is_signed_in():
+        server_client.auth.sign_in(authentication)
 
     start_date = config['start_date'] if 'start_date' in config else None
     selected_stream_ids = get_selected_streams(catalog)
+    validate_dependencies(selected_stream_ids)
 
     for stream in catalog['streams']:
         stream_id = stream['tap_stream_id']
-        stream_schema = stream['schema']
-        mdata = stream['metadata']
         if not SYNC_FUNCTIONS.get(stream_id):
             continue
         if stream_id in selected_stream_ids:
-            singer.write_schema(stream_id, stream_schema, stream['key_properties'])
+            singer.write_schema(stream_id, stream['schema'], stream['key_properties'])
             sync_func = SYNC_FUNCTIONS[stream_id]
             sub_stream_ids = SUB_STREAMS.get(stream_id, None)
             if not sub_stream_ids:
-                state = sync_func(stream_schema, server, authentication, state, mdata, start_date)
+                state = sync_func(stream['schema'], server_client, state, stream['metadata'], start_date)
             else:
-                stream_schemas = {stream_id: stream_schema}
+                stream_schemas = {stream_id: stream['schema']}
                 for sub_stream_id in sub_stream_ids:
                     if sub_stream_id in selected_stream_ids:
                         sub_stream = get_stream_from_catalog(sub_stream_id, catalog)
                         stream_schemas[sub_stream_id] = sub_stream['schema']
                         singer.write_schema(sub_stream_id, sub_stream['schema'], sub_stream['key_properties'])
-                state = sync_func(stream_schema, server, authentication, state, mdata, start_date)
+                state = sync_func(stream_schemas, server_client, state, stream['metadata'], start_date)
             singer.write_state(state)
 
 
@@ -265,7 +287,7 @@ def do_sync(config, state, catalog):
 def main():
     args = singer.utils.parse_args(REQUIRED_CONFIG_KEYS)
     if args.discover:
-        discover(args.config)
+        discover()
     else:
         catalog = args.catalog.to_dict() if args.catalog else get_catalog()
         do_sync(args.config, args.state, catalog)
