@@ -9,23 +9,21 @@ def get_group_details(group):
         'minimum_site_role': group.minimum_site_role,
         'name': group.name,
         'tag_name': group.tag_name,
-        'users': [user for user in group.users]
+        'users': list(group.users)
     }
 
 
-def get_all_groups(server):
-    all_groups, _ = server.groups.get()
+def get_all_groups(server_client):
+    all_groups, _ = server_client.groups.get()
     for group in all_groups:
-        server.groups.populate_users(group)
+        server_client.groups.populate_users(group)
     return all_groups
 
 
-def get_all_group_details(server, authentication):
+def get_all_group_details(server_client):
     groups = []
     users = []
-    if not server.is_signed_in():
-        server.auth.sign_in(authentication)
-    all_groups = get_all_groups(server=server)
+    all_groups = get_all_groups(server_client=server_client)
     for group in all_groups:
         group_details = get_group_details(group=group)
         groups.append(group_details)
@@ -33,7 +31,6 @@ def get_all_group_details(server, authentication):
             if user.id not in set(user['id'] for user in users):
                 users.append(get_user_details(user=user))
         group_details.pop('users')
-    server.auth.sign_out()
     return {
         'groups': groups,
         'users': users,
